@@ -7,14 +7,22 @@ import (
 
 type Version string
 
-func (me Version) compareTo(other string) int {
+func (me Version) compareTo(other Version) int {
 	var (
 		meTab    = strings.Split(string(me), ".")
-		otherTab = strings.Split(other, ".")
+		otherTab = strings.Split(string(other), ".")
 	)
-	for i, s := range meTab {
+
+	max := len(meTab)
+	if len(otherTab) > max {
+		max = len(otherTab)
+	}
+	for i := 0; i < max; i++ {
 		var meInt, otherInt int
-		meInt, _ = strconv.Atoi(s)
+
+		if len(meTab) > i {
+			meInt, _ = strconv.Atoi(meTab[i])
+		}
 		if len(otherTab) > i {
 			otherInt, _ = strconv.Atoi(otherTab[i])
 		}
@@ -25,28 +33,25 @@ func (me Version) compareTo(other string) int {
 			return -1
 		}
 	}
-	if len(otherTab) > len(meTab) {
-		return -1
-	}
 	return 0
 }
 
-func (me Version) LessThan(other string) bool {
+func (me Version) LessThan(other Version) bool {
 	return me.compareTo(other) == -1
 }
 
-func (me Version) LessThanOrEqualTo(other string) bool {
+func (me Version) LessThanOrEqualTo(other Version) bool {
 	return me.compareTo(other) <= 0
 }
 
-func (me Version) GreaterThan(other string) bool {
+func (me Version) GreaterThan(other Version) bool {
 	return me.compareTo(other) == 1
 }
 
-func (me Version) GreaterThanOrEqualTo(other string) bool {
+func (me Version) GreaterThanOrEqualTo(other Version) bool {
 	return me.compareTo(other) >= 0
 }
 
-func (me Version) Equal(other string) bool {
+func (me Version) Equal(other Version) bool {
 	return me.compareTo(other) == 0
 }
